VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "FileFunctions"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Private Declare Function WaitForSingleObject Lib "kernel32" (ByVal hHandle As Long, ByVal dwMilliseconds As Long) As Long
Private Declare Function CreateProcessA Lib "kernel32" (ByVal lpApplicationName As Long, ByVal lpCommandLine As String, ByVal lpProcessAttributes As Long, ByVal lpThreadAttributes As Long, ByVal bInheritHandles As Long, ByVal dwCreationFlags As Long, ByVal lpEnvironment As Long, ByVal lpCurrentDirectory As Long, lpStartupInfo As STARTUPINFO, lpProcessInformation As PROCESS_INFORMATION) As Long
Private Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long

Private Const NORMAL_PRIORITY_CLASS = &H20&
Private Const INFINITE = -1&

Private Type STARTUPINFO
   cb As Long
   lpReserved As String
   lpDesktop As String
   lpTitle As String
   dwX As Long
   dwY As Long
   dwXSize As Long
   dwYSize As Long
   dwXCountChars As Long
   dwYCountChars As Long
   dwFillAttribute As Long
   dwFlags As Long
   wShowWindow As Integer
   cbReserved2 As Integer
   lpReserved2 As Long
   hStdInput As Long
   hStdOutput As Long
   hStdError As Long
End Type

Private Type PROCESS_INFORMATION
   hProcess As Long
   hThread As Long
   dwProcessID As Long
   dwThreadID As Long
End Type

' Runs command line and returns only when shelled
' process has terminated - returns number of seconds
' process was alive
'
Public Function ShellAndWait(CommandLine)
Attribute ShellAndWait.VB_Description = "Shells a command line and suspends execution until the shelled process returns"

    Dim proc As PROCESS_INFORMATION
    Dim Start As STARTUPINFO
    Dim t0 As Single
    
    t0 = Timer
    
    ' Initialize the STARTUPINFO structure:
    Start.cb = Len(Start)
    Start.wShowWindow = 0
    
    ' Start the shelled application:
    ret& = CreateProcessA(0&, CommandLine, 0&, 0&, 1&, NORMAL_PRIORITY_CLASS, 0&, 0&, Start, proc)
    
    ' Wait for the shelled application to finish:
    ret& = WaitForSingleObject(proc.hProcess, INFINITE)
    ret& = CloseHandle(proc.hProcess)

    ShellAndWait = Timer - t0

End Function

' returns true if specified file or volume exists
'
Public Function Exists(Name) As Boolean
Attribute Exists.VB_Description = "Returns True if the specified file, directory or volume exists"
    Exists = (Dir(Name, 63) > "")
End Function

' attempts to identify mime type of file
'
Public Function FileType(FileName, Optional Content) As String
Attribute FileType.VB_Description = "Attempts to determine the MIME type of a file"
    Dim Identify As String
    ' only look at first 1k of file
    If IsMissing(Content) Then
        Content = ReadFile(FileName, 1024)
    Else
        Content = Left(Content, 1024)
    End If
    ' identify
    Identify = IdentifyBySignature(CStr(Content))
    If Identify = "application/octet-stream" Then Identify = IdentifyByExtension(FileName)
    If Identify = "application/octet-stream" Then Identify = IdentifyByContent(CStr(Content))
    FileType = Identify
End Function

' returns name of a unique temporary file, in folder, if specified
'
Public Function TempFile(Optional ByVal Folder As String = vbNullString) As String
Attribute TempFile.VB_Description = "Returns a unique temporary file name"
    Dim Temp As String
    If Folder = vbNullString Then Folder = Environ("temp")
    If Right(Folder, 1) <> "\" Then Folder = Folder & "\"
    Do
        Temp = Folder & (100 * Timer)
    Loop Until Dir(Temp) = ""
    TempFile = Temp
End Function

' returns contents of file in a string (only ByteCount chars if parameter specfied)
'
Public Function ReadFile(ByVal FileName As String, Optional ByVal ByteCount As Long = 0) As String
Attribute ReadFile.VB_Description = "Returns the contents of the specified file"
On Error GoTo errhandler
    Dim f As Integer
    ReadFile = vbNullString
    f = FreeFile
    Open FileName For Binary As #f
    If ByteCount = 0 Or ByteCount > LOF(f) Then
        ReadFile = Input(LOF(f), #f)
    Else
        ReadFile = Input(ByteCount, #f)
    End If
errhandler:
    Close #f
End Function

' writes data to file; if file not specified, uses temp file
' returns name of file written to
'
Public Function WriteFile(ByRef data As String, Optional ByVal FileName As String = vbNullString) As String
Attribute WriteFile.VB_Description = "Writes the specified data into the specified (or a temporary) file; returns file name"
    Dim f As Integer
    If FileName = vbNullString Then FileName = TempFile
    f = FreeFile
    Open FileName For Output As #f
    Print #f, data
    Close #f
    WriteFile = FileName
End Function

' ----- PRIVATE FUNCTIONS -----

Private Function IdentifyBySignature(ByRef Content As String) As String
    Dim s As New StringFunctions
    IdentifyBySignature = "application/octet-stream"
    Signature = Left(Content, 32)
    If Mid(Signature, 7, 5) = "JFIF" & vbNullChar Then IdentifyBySignature = "image/jpeg"
    If Left(Signature, 3) = "GIF" And IsNumeric(Mid(Signature, 4, 2)) And Mid(Signature, 6, 1) >= "a" And Mid(Signature, 6, 1) <= "z" Then IdentifyBySignature = "image/gif"
    If Left(Signature, 2) = "BM" And Mid(Signature, 6, 1) = vbNullChar Then IdentifyBySignature = "image/bmp"
    If StrComp(Left(s.LTrimWS(Signature), 6), "<html>", vbTextCompare) = 0 Or StrComp(Left(s.LTrimWS(Signature), 14), "<!doctype html", vbTextCompare) = 0 Then IdentifyBySignature = "text/html"
End Function

Private Function IdentifyByExtension(ByVal FileName As String) As String
    IdentifyByExtension = "application/octet-stream"
    FileName = LCase(Mid(FileName, InStrRev(FileName, ".") + 1))
    Select Case FileName
    Case "txt", "text", "asc": IdentifyByExtension = "text/plain"
    Case "tab", "tsv": IdentifyByExtension = "text/tab-separated-values"
    Case "csv": IdentifyByExtension = "text/comma-separated-values"
    Case "jpg", "jpe", "jfi", "jfif", "jpeg": IdentifyByExtension = "image/jpeg"
    Case "rtf": IdentifyByExtension = "text/rtf"
    Case "xml": IdentifyByExtension = "text/xml"
    Case "html", "htm", "plg": IdentifyByExtension = "text/html"
    Case "css": IdentifyByExtension = "text/css"
    Case "zip": IdentifyByExtension = "application/zip"
    Case "sgml": IdentifyByExtension = "application/sgml"
    Case "gif": IdentifyByExtension = "image/gif"
    Case "tif", "tiff": IdentifyByExtension = "image/tiff"
    Case "png": IdentifyByExtension = "image/png"
    Case "mpg", "mpeg": IdentifyByExtension = "video/mpeg"
    Case "avi": IdentifyByExtension = "video/avi"
    Case "qt": IdentifyByExtension = "video/quicktime"
    Case "wav": IdentifyByExtension = "audio/wav"
    Case "mp3": IdentifyByExtension = "audio/mp3"
    Case "bmp": IdentifyByExtension = "image/bmp"
    Case "pcx": IdentifyByExtension = "image/pcx"
    Case "iff", "ilb", "ilbm": IdentifyByExtension = "image/iff-ilbm"
    Case "url": IdentifyByExtension = "text/x-windows-url"
    Case "ico": IdentifyByExtension = "image/x-windows-icon"
    Case "cur": IdentifyByExtension = "image/x-windows-cursor"
    Case "ani": IdentifyByExtension = "image/x-windows-animated-cursor"
    Case Else: IdentifyByExtension = "application/octet-stream"
    End Select
End Function

Private Function IdentifyByContent(ByRef Content As String) As String
    Dim s As New StringFunctions
    IdentifyByContent = IIf(s.IsBinary(Content), "application/octet-stream", "text/plain")
End Function


