VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "StringFunctions"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Public Enum VBConversion
    vbUpperCase = 1& ' Converts the string to uppercase characters.
    vbLowerCase = 2& ' Converts the string to lowercase characters.
    vbProperCase = 3& ' Converts the first letter of every word in string to uppercase.
    vbWide = 4& ' Converts narrow (single-byte) characters in string to wide (double-byte) characters. Applies to Far Eastlocales.
    vbNarrow = 8& ' Converts wide (double-byte) characters in string to narrow (single-byte) characters. Applies to Far East locales.
    vbKatakana = 16& ' Converts Hiragana characters in string to Katakana characters. Applies to Japan only.
    vbHiragana = 32& ' Converts Katakana characters in string to Hiragana characters. Applies to Japan only.
    vbUnicode = 64& ' Converts the string toUnicode using the default code page of the system.
    vbFromUnicode = 128& ' Converts the string from Unicode to the default code page of the system.
    vb7bit = 256&
    vb8Bit = 257&
    vbQuotedPrintable = 258&
    vbFromQuotedPrintable = 259&
    vbBase64 = 260&
    vbFromBase64 = 261&
    vbUUEncode = 262&
    vbUUDecode = 263&
    vbReversed = 264&
    vbByteArray = 265&
    vbWordArray = 266&
    vbEBCDIC = 267&
    vbFromEBCDIC = 268&
End Enum

Public Function Restring(String1, Conversion As VBConversion, Optional LocaleID As Long) As Variant
Attribute Restring.VB_Description = "Allows a selection of string conversions (extension of StrConv function)"
    If Len(String1) > 0 Then
        Select Case Conversion
        Case vbUpperCase
            Restring = StrConv(String1, vbUpperCase, LocaleID)
        Case vbLowerCase
            Restring = StrConv(String1, vbLowerCase, LocaleID)
        Case vbProperCase
            Restring = StrConv(String1, vbProperCase, LocaleID)
        Case vbWide
            Restring = StrConv(String1, vbWide, LocaleID)
        Case vbNarrow
            Restring = StrConv(String1, vbNarrow, LocaleID)
        Case vbKatakana
            Restring = StrConv(String1, vbKatakana, LocaleID)
        Case vbHiragana
            Restring = StrConv(String1, vbHiragana, LocaleID)
        Case vbUnicode
            Restring = StrConv(String1, vbUnicode, LocaleID)
        Case vbFromUnicode
            Restring = StrConv(String1, vbFromUnicode, LocaleID)
        Case vb7bit
            Dim Encoded() As Byte, ch As Long
            Encoded() = StrConv(String1, vbFromUnicode, LocaleID)
            For ch = LBound(Encoded) To UBound(Encoded)
                If Encoded(ch) >= 128 Then Encoded(ch) = Encoded(ch) And 127
            Next
            Restring = StrConv(Encoded(), vbUnicode, LocaleID)
        Case vb8Bit
            Restring = StrConv(StrConv(String1, vbFromUnicode, LocaleID), vbUnicode, LocaleID)
        Case vbQuotedPrintable
            Restring = EncodeQ(String1)
        Case vbFromQuotedPrintable
            Restring = DecodeQ(String1)
        Case vbBase64
            Restring = EncodeB(String1)
        Case vbFromBase64
            Restring = DecodeB(String1)
        Case vbUUEncode
            Restring = EncodeU(String1)
        Case vbUUDecode
            Restring = DecodeU(String1)
        Case vbReversed
            Restring = StrReverse(String1)
        Case vbByteArray
            Dim ba() As Byte
            ba() = StrConv(String1, vbFromUnicode, LocaleID)
            Restring = ba()
        Case vbWordArray
            Dim wa() As Integer
            ReDim wa(0 To Len(String1) - 1) As Integer
            CopyMemory VarPtr(wa(LBound(wa))), StrPtr(String1), LenB(String1)
            Restring = wa()
        Case vbEBCDIC
            Restring = EncodeE(String1)
        Case vbFromEBCDIC
            Restring = DecodeE(String1)
        End Select
    Else
        Restring = vbNullString
    End If
End Function

Public Function UnLeft(String1, Length As Long)
Attribute UnLeft.VB_Description = "Removes a specified number of characters from the left side of a string and returns the remainder"
    UnLeft = Mid(String1, Length + 1)
End Function

Public Function UnRight(String1, Length As Long)
Attribute UnRight.VB_Description = "Removes a specified number of characters from the right side of a string and returns the remainder"
    UnRight = Left(String1, Len(String1) - Length)
End Function

Public Function UnMid(String1, Start As Long, Optional Length)
Attribute UnMid.VB_Description = "Removes a specified number of characters from the middle of a string"
    If IsMissing(Length) Then
        UnMid = Left(String1, Start - 1)
    Else
        UnMid = Left(String1, Start - 1) & Mid(String1, Start + Length)
    End If
End Function

' opposite of split function - combines array into
' string using delimiter character (default = <space>)
'
Public Function Glue(Array1(), Optional Delimiter)
    Dim delim As String, i As Integer, OutStr As String
    If IsMissing(Delimiter) Then delim = " " Else delim = Delimiter
    OutStr = vbNullString
    For i = LBound(Array1) To UBound(Array1)
        OutStr = OutStr & CStr(Array1(i)) & IIf(i < UBound(Array1), delim, vbNullString)
    Next
    Glue = OutStr
End Function

' returns true if string contains any characters with bit 7 set
' (ascii value >= 128)
'
Public Function IsBinary(String1) As Boolean
Attribute IsBinary.VB_Description = "Returns True if string contains characters with bit 7 set"
    Dim Bytes() As Byte, isBin As Boolean, i As Long
    Bytes() = StrConv(CStr(String1), vbFromUnicode)
    isBin = False
    i = LBound(Bytes)
    While i <= UBound(Bytes) And Not isBin
        If Bytes(i) >= 128 Then isBin = True
        i = i + 1
    Wend
    IsBinary = isBin
End Function

' trims all whitespace characters from left and right of string
'
Public Function TrimWS(ByVal String1 As String) As String
Attribute TrimWS.VB_Description = "Returns a copy of a string without leading and trailing control characters"
    TrimWS = RTrimWS(LTrimWS(String1))
End Function

' trims all whitespace characters from left of string
'
Public Function LTrimWS(ByVal String1 As String) As String
Attribute LTrimWS.VB_Description = "Returns a copy of a string without leading control characters"
    While String1 > "" And Asc(String1 & " ") <= 32
        String1 = Mid(String1, 2)
    Wend
    LTrimWS = String1
End Function

' trims all whitespace characters from right of string
'
Public Function RTrimWS(ByVal String1 As String) As String
Attribute RTrimWS.VB_Description = "Returns a copy of a string without trailing control characters"
    While String1 > "" And Asc(Right(" " & String1, 1)) <= 32
        String1 = Left(String1, Len(String1) - 1)
    Wend
    RTrimWS = String1
End Function

' trims quotes from string
'
Public Function TrimQuotes(ByVal String1 As String) As String
Attribute TrimQuotes.VB_Description = "Returns a copy of a string without quotes"
    String1 = TrimWS(String1)
    While Left(String1, 1) = Chr(34)
        String1 = Mid(String1, 2)
    Wend
    While Right(String1, 1) = Chr(34)
        String1 = Left(String1, Len(String1) - 1)
    Wend
    TrimQuotes = String1
End Function

' ----- PRIVATE FUNCTIONS -----

' returns base-64 encoded string
'
Private Function EncodeB(ByVal X As String)

    X = Replace(TrimWS(X), vbCrLf, "")
    
    Dim Lookup()
    Lookup = Array(65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, _
                            81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 97, 98, 99, 100, 101, 102, _
                            103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, _
                            119, 120, 121, 122, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 43, 47)
    
    Dim Size As Long
    Size = (Len(X) + 2) \ 3
    
    Dim Decoded() As Byte
    ReDim Decoded(0 To 3 * Size - 1) As Byte
    Decoded() = StrConv(X, vbFromUnicode)
    
    Dim Encoded() As Byte
    ReDim Encoded(0 To 4 * Size - 1) As Byte
    
    Dim Chunk As Long
    Dim Chunk3 As Long
    Dim Chunk4 As Long

    For Chunk = 0 To Size - 1
        Chunk3 = 3 * Chunk
        Chunk4 = 4 * Chunk
        Encoded(Chunk4) = Lookup(Decoded(Chunk3) \ 4)
        If Chunk3 + 1 <= UBound(Decoded) Then
            Encoded(Chunk4 + 1) = Lookup(16 * (Decoded(Chunk3) And 3) Or Decoded(Chunk3 + 1) \ 16)
            If Chunk3 + 2 <= UBound(Decoded) Then
                Encoded(Chunk4 + 2) = Lookup(4 * (Decoded(Chunk3 + 1) And 15) Or Decoded(Chunk3 + 2) \ 64)
                Encoded(Chunk4 + 3) = Lookup(Decoded(Chunk3 + 2) And 63)
            Else
                Encoded(Chunk4 + 2) = Lookup(4 * (Decoded(Chunk3 + 1) And 15))
                Encoded(Chunk4 + 3) = 61
            End If
        Else
            Encoded(Chunk4 + 1) = Lookup(16 * (Decoded(Chunk3) And 3))
            Encoded(Chunk4 + 2) = 61
            Encoded(Chunk4 + 3) = 61
        End If
    Next

    EncodeB = StrConv(Encoded(), vbUnicode)

End Function

' returns text decoded from base-64 string
'
Private Function DecodeB(ByVal X As String)
    
    X = Replace(TrimWS(X), vbCrLf, "")
    
    Dim Lookup()
    Lookup() = Array(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
                   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 62, 0, 0, 0, 63, _
                   52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 0, 0, 0, 0, 0, 0, _
                   0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, _
                   15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 0, 0, 0, 0, 0, _
                   0, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, _
                   41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 0, 0, 0, 0, 0)
    
    Dim Size As Long
    Size = Len(X) \ 4
    
    Dim Encoded() As Byte
    ReDim Encoded(0 To 4 * Size - 1) As Byte
    Encoded() = StrConv(X, vbFromUnicode)
    
    Dim Decoded() As Byte
    ReDim Decoded(0 To 3 * Size - 1) As Byte
    
    Dim Chunk As Long
    Dim Chunk3 As Long
    Dim Chunk4 As Long
    
    Dim Temp(0 To 3) As Byte
    
    For Chunk = 0 To Size - 1
        Chunk3 = 3 * Chunk
        Chunk4 = 4 * Chunk
        Temp(0) = Lookup(Encoded(Chunk4))
        Temp(1) = Lookup(Encoded(Chunk4 + 1))
        Temp(2) = Lookup(Encoded(Chunk4 + 2))
        Temp(3) = Lookup(Encoded(Chunk4 + 3))
        Decoded(Chunk3) = 4 * Temp(0) Or Temp(1) \ 16
        Decoded(Chunk3 + 1) = 16 * (Temp(1) And 15) Or Temp(2) \ 4
        Decoded(Chunk3 + 2) = 64 * (Temp(2) And 3) Or Temp(3)
    Next
    
    DecodeB = StrConv(Decoded(), vbUnicode)
    
End Function

Private Function EncodeQ(ByVal X As String) As String
    ' remove '=' signs
    If InStr(1, X, "=") > 0 Then X = Replace(X, "=", "=3D")
    ' remove tab & space at end of line
    X = X & vbCrLf
    If InStr(1, X, vbTab & vbCrLf) > 0 Then X = Replace(X, vbTab & vbCrLf, "=09" & vbCrLf)
    If InStr(1, X, " " & vbCrLf) > 0 Then X = Replace(X, " " & vbCrLf, "=20" & vbCrLf)
    X = UnRight(X, 2)
    ' other chars
    For i = 0 To 8
        If InStr(1, X, Chr(i)) > 0 Then X = Replace(X, Chr(i), "=" + Right("0" + Hex(i), 2))
    Next
    For i = 10 To 31
        If InStr(1, X, Chr(i)) > 0 Then X = Replace(X, Chr(i), "=" + Right("0" + Hex(i), 2))
    Next
    For i = 127 To 255
        If InStr(1, X, Chr(i)) > 0 Then X = Replace(X, Chr(i), "=" + Right("0" + Hex(i), 2))
    Next
    ' ignore lines not > 76 chars rule
    EncodeQ = X
End Function

Private Function DecodeQ(ByVal X As String) As String
    Dim eq As Long
    X = Replace(X, "=" & vbCrLf, "")
    Do
        eq = InStr(eq + 1, X, "=")
        If eq > 0 Then X = Left(X, eq - 1) + Chr(Val("&H" & Mid(X, eq + 1, 2))) + Mid(X, eq + 3)
    Loop Until eq = 0
    DecodeQ = X
End Function

Private Function EncodeU(ByVal X As String)
    EncodeU = X
End Function

Private Function DecodeU(ByVal X As String)
    DecodeU = X
End Function

Private Function EncodeE(ByVal X As String)
    Dim EBCDIC(), i As Integer, Encoded() As Byte
    EBCDIC() = Array(0, 1, 2, 3, 55, 45, 46, 47, 22, 5, 37, 11, 12, 13, 14, 15, _
                                16, 17, 18, 19, 60, 61, 50, 38, 18, 19, 63, 39, 28, 29, 30, 31, _
                                64, 90, 127, 123, 91, 108, 80, 125, 77, 93, 92, 78, 107, 96, 75, 97, _
                                240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 122, 94, 76, 126, 110, 111, _
                                124, 193, 194, 195, 196, 197, 198, 199, 200, 201, 209, 210, 211, 212, 213, 214, _
                                215, 216, 217, 226, 227, 228, 229, 230, 231, 232, 233, 176, 224, 187, 186, 109, _
                                185, 129, 130, 131, 132, 133, 134, 135, 136, 137, 145, 146, 147, 148, 149, 150, _
                                151, 152, 153, 162, 163, 164, 165, 166, 167, 168, 169, 192, 106, 208, 161, 7, _
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
    Encoded() = StrConv(X, vbFromUnicode)
    For i = LBound(Encoded) To UBound(Encoded)
        Encoded(i) = EBCDIC(Encoded(i))
    Next
    EncodeE = StrConv(Encoded(), vbUnicode)
End Function

Private Function DecodeE(ByVal X As String)
    Dim EBCDIC(), i As Integer, Decoded() As Byte
    EBCDIC() = Array(0, 1, 2, 3, 0, 9, 0, 127, 0, 0, 0, 11, 12, 13, 14, 15, _
                                16, 17, 18, 19, 0, 0, 8, 0, 24, 25, 26, 27, 28, 29, 30, 31, _
                                0, 0, 0, 0, 0, 10, 23, 27, 0, 0, 0, 0, 0, 5, 6, 7, _
                                0, 0, 22, 0, 0, 0, 0, 4, 0, 0, 0, 0, 20, 21, 0, 26, _
                                32, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 46, 60, 40, 43, 0, _
                                38, 0, 0, 0, 0, 0, 0, 0, 0, 0, 33, 36, 42, 41, 59, 0, _
                                45, 47, 0, 0, 0, 0, 0, 0, 0, 0, 124, 44, 37, 95, 62, 63, _
                                0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 58, 35, 64, 39, 61, 34, _
                                0, 97, 98, 99, 100, 101, 102, 103, 104, 105, 0, 0, 0, 0, 0, 0, _
                                0, 106, 107, 108, 109, 110, 111, 112, 113, 114, 0, 0, 0, 0, 0, 0, _
                                0, 126, 115, 116, 117, 118, 119, 120, 121, 122, 0, 0, 0, 0, 0, 0, _
                                91, 0, 0, 0, 0, 0, 0, 0, 0, 96, 94, 93, 0, 0, 0, 0, _
                                123, 65, 66, 67, 68, 69, 70, 71, 72, 73, 0, 0, 0, 0, 0, 0, _
                                125, 74, 75, 76, 77, 78, 79, 80, 81, 82, 0, 0, 0, 0, 0, 0, _
                                92, 0, 83, 84, 85, 86, 87, 88, 89, 90, 0, 0, 0, 0, 0, 0, _
                                48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 0, 0, 0, 0, 0, 0)
    Decoded() = StrConv(X, vbFromUnicode)
    For i = LBound(Decoded) To UBound(Decoded)
        Decoded(i) = EBCDIC(Decoded(i))
    Next
    DecodeE = StrConv(Decoded(), vbUnicode)
End Function


